#include <SnoopAdapter>
#include <SnoopTcpData>
#include <SnoopTcpBlock>
#include <iostream>
using namespace std;

int sendForwardBlock(pcap* pcap, SnoopPacket* packet, UINT8 flag, QByteArray msg = "")
{
  return -1; // gilgil temp
}

int sendBackwardBlock(pcap* pcap, SnoopPacket* packet, UINT8 flag, QByteArray msg = "")
{
  if (packet->proto != IPPROTO_TCP)
  {
    LOG_ERROR("packet->proto is %d", (int)packet->proto);
    return -1;
  }

	int bufSize = sizeof(ETH_HDR) + sizeof(IP_HDR) + sizeof(TCP_HDR) + msg.length();
	QByteArray ba; ba.resize(bufSize); char* buf = ba.data();

	ETH_HDR* ethHdr = (ETH_HDR*) &buf[0];
	IP_HDR*  ipHdr  = (IP_HDR*)  &buf[sizeof(ETH_HDR)];
	TCP_HDR* tcpHdr = (TCP_HDR*) &buf[sizeof(ETH_HDR) + sizeof(IP_HDR)];
	char*    data   = (char*)    &buf[sizeof(ETH_HDR) + sizeof(IP_HDR) + sizeof(TCP_HDR)];

  //
  // Ethernet Header
  //
  ethHdr->ether_dhost = packet->ethHdr->ether_shost;
  ethHdr->ether_shost = packet->ethHdr->ether_dhost;
  ethHdr->ether_type  = packet->ethHdr->ether_type;

	//
	// IP Header
	//
	ipHdr->ip_hl  = 5;
	ipHdr->ip_v   = 4;
	ipHdr->ip_tos = 44;
	ipHdr->ip_len = htons(sizeof(IP_HDR) + sizeof(TCP_HDR) + msg.length());
	ipHdr->ip_id  = 0;
	ipHdr->ip_off = 0;
	ipHdr->ip_ttl = 255;
	ipHdr->ip_p   = IPPROTO_TCP;
	ipHdr->ip_src = packet->ipHdr->ip_dst;
	ipHdr->ip_dst = packet->ipHdr->ip_src;

	//
	// TCP Header
	//
	int tcpDataLen;
	if (!SnoopTcp::isData(packet->ipHdr, packet->tcpHdr, NULL, &tcpDataLen)) tcpDataLen = 0;
	int flagAddLen = ((packet->tcpHdr->th_flags & (TH_SYN | TH_FIN))) ? 1 : 0;
	UINT32 newSeq = ntohl(packet->tcpHdr->th_seq) + tcpDataLen + flagAddLen;

	tcpHdr->th_sport = packet->tcpHdr->th_dport;
	tcpHdr->th_dport = packet->tcpHdr->th_sport;
	tcpHdr->th_seq   = packet->tcpHdr->th_ack;
	tcpHdr->th_ack   = htonl(newSeq);
	tcpHdr->th_off   = sizeof(TCP_HDR) / sizeof(UINT32);
	tcpHdr->th_flags = flag | TH_ACK;
	tcpHdr->th_x2    = 0;
	tcpHdr->th_win   = 0;
	tcpHdr->th_urp   = 0;

	//
	// Data
	//
	memcpy(data, msg.data(), msg.length());

	//
	// Checksum
	//
	tcpHdr->th_sum = htons(SnoopTcp::checksum(ipHdr, tcpHdr));
	ipHdr->ip_sum  = htons(SnoopIp::checksum(ipHdr));

	int res = pcap_sendpacket(pcap, (const u_char*)buf, bufSize);
	if (res < 0)
	{
		LOG_ERROR("pcap_sendpacket return %d", res);
	}
	return res;
}

int main()
{
  // QCoreApplication a(argc, argv);

  SnoopAdapter adapter;
  adapter.autoRead = false;
  adapter.filter   = "port 80";

  if (!adapter.open())
  {
    cout << adapter.error.msg << endl;
    return -1;
  }

  while (true)
  {
    SnoopPacket packet;
    int readLen = adapter.read(&packet);
    if (readLen == 0) continue; // timeout
    if (readLen == -1) break;   // end of capturing

    if (!SnoopTcp::parseAll(&packet)) continue;
    if (packet.tcpHdr->th_flags & (TH_RST | TH_FIN)) continue;

    sendBackwardBlock(adapter.m_pcap, &packet, TH_RST);
    LOG_DEBUG("TCP connection killed");
  }

  //return a.exec();
  return 0;
}
